use std::fs::File;
use std::io::{self, BufRead};
use std::path::Path;

fn get_distance(a: (i64, i64), b: (i64, i64)) -> i64 {
    (a.0 - b.0).abs() + (a.1 - b.1).abs()
}

fn main() {
    let file_path = String::from("../aoc_15a/input");
    println!("In file {}", file_path);
    let mut sensors: Vec<(i64, i64)> = vec![];
    let mut beacon: Vec<(i64, i64)> = vec![];
    let mut distances: Vec<i64> = vec![];

    if let Ok(lines) = read_lines(file_path) {
        // Consumes the iterator, returns an (Optional) String
        for line in lines {
            if let Ok(ip) = line {
                let split = ip.split(" ")
                    .enumerate()
                    .filter(|&(i, _)| i == 2 || i == 3 || i == 8 || i == 9)
                    .map(|(i, v)| v[2..v.len() - 1 + (i == 9) as usize].parse::<i64>().unwrap())
                    .collect::<Vec<i64>>();
                sensors.push((split[0], split[1]));
                beacon.push((split[2], split[3]));
                distances.push((split[0] - split[2]).abs() + (split[1] - split[3]).abs());
            }
        }
    }

    let mut tool: Vec<(i64, i64, i64)> = vec![];
    for (i, _) in sensors.iter().enumerate() {
        for j in i + 1..sensors.len() {
            let distance = (sensors[i].0 - sensors[j].0).abs() + (sensors[i].1 - sensors[j].1).abs();
            let simple = (distance - distances[i] - distances[j]).abs();
            if simple == 2 {
                println!("Found something between {} and {}, {} {} {} {}", i, j, simple, distance, distances[i], distances[j]);
                let mut x = 0;
                let y = (sensors[i].1 + sensors[j].1) / 2;
                let mut d1 = (get_distance(sensors[i], (x, y)) - distances[i]).abs();
                let mut d2 = (get_distance(sensors[j], (x, y)) - distances[j]).abs();
                while d1 + d2 > 2 {
                    d2 = (get_distance(sensors[j], (x, y)) - distances[j]).abs();
                    d1 = (get_distance(sensors[i], (x, y)) - distances[i]).abs();
                    x += 1;
                }

                println!("{} {} {} {}", d1, d2, x, y);
                d2 = (get_distance(sensors[j], (x + 1, y + 1)) - distances[j]).abs();
                d1 = (get_distance(sensors[i], (x + 1, y + 1)) - distances[i]).abs();
                if d1 + d2 == 2 {
                    tool.push((x, y, 1));
                }
                else {
                    tool.push((x, y, -1));
                }
                println!("{} {} {} {}", d1, d2, x, y);
            }
        }
    }

    println!("{:#?}", tool);
    let dir = tool[0].2;
    let mut a = (tool[0].0 - 3000000, tool[0].1 - 3000000 * dir);
    let b = (tool[1].0, tool[1].1);
    while (a.0 - b.0).abs() != (a.1 - b.1).abs() {
        a.0 += 1;
        a.1 += dir;
    }

    println!("{:?}", a);

    println!("{} {}", distances[6], get_distance(sensors[6], a));
    println!("{} {}", distances[29], get_distance(sensors[29], a));
    println!("{} {}", distances[24], get_distance(sensors[24], a));
    println!("{} {}", distances[31], get_distance(sensors[31], a));

    println!("{}", a.0 * 4000000 + a.1);
}

fn read_lines<P>(filename: P) -> io::Result<io::Lines<io::BufReader<File>>>
where P: AsRef<Path>, {
    let file = File::open(filename)?;
    Ok(io::BufReader::new(file).lines())
}
